@@ -6,8 +6,8 @@ from models import GroupMigration
 
 class GroupMigrationInline(admin.TabularInline):
     model = GroupMigration
-    fields = ('group','time','offset','hidden','members_count','members_entered_count','members_left_count')
-    readonly_fields = ('group','time','offset','members_count','members_entered_count','members_left_count')
+    fields = ('id','group','time','offset','hidden','members_count','members_entered_count','members_left_count')
+    readonly_fields = fields
     ordering = ('-time',)
     extra = 0
     can_delete = False
@@ -26,4 +26,4 @@ class GroupMigrationAdmin(VkontakteModelAdmin):
 
 admin.site.unregister(Group)
 admin.site.register(Group, GroupAdmin)
-admin.site.register(GroupMigration, GroupMigrationAdmin)
\ No newline at end of file
+admin.site.register(GroupMigration, GroupMigrationAdmin)
