@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 from celery.task import Task
-from vkontakte_groups_migration.models import GroupMigration
+from vkontakte_groups_migration.models import GroupMigration, update_group_users
 
 class VkontakteGroupUpdateUsersM2M(Task):
 
@@ -9,7 +9,7 @@ class VkontakteGroupUpdateUsersM2M(Task):
         logger = self.get_logger(**kwargs)
         logger.info(u'VK group "%s" users m2m relations updating started' % stat.group)
         try:
-            stat.update_users_relations()
+            update_group_users(stat)
             logger.info(u'VK group "%s" users m2m relations succesfully updated' % stat.group)
         except:
             logger.error(u'Unknown error while updating users m2m relations of VK group "%s"' % stat.group)
